<footer class="footer">
 <div style="display:flex; justify-content: space-between;">
  <div>Made with ❤️ by the dracut community</div>
  <div>This site is built with <a href="https://antora.org/">Antora</a>.</div>
 </div>
</footer>
